/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.api;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.SocketException;
import java.util.List;
import java.util.Map;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.commons.log.FlowLogData;
import org.ow2.petals.commons.log.Level;
import org.ow2.petals.commons.log.LogData;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.commons.log.TraceCode;
import org.ow2.petals.component.api.ServiceConfiguration.ServiceType;
import org.ow2.petals.log.handler.TestHandler;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public abstract class AbstractComponentTest {

    protected Component componentUnderTest;

    protected ComponentConfiguration componentConfiguration;

    protected TestHandler testHandler;

    protected Logger logger;

    @Before
    public void before() {
        this.testHandler = new TestHandler();
        this.logger = TestHandler.getLoggerWithTestHandler("testLogger", testHandler);

        this.componentConfiguration = createComponentConfiguration(this.logger);
        this.componentUnderTest = createComponentUnderTest(componentConfiguration);

    }

    @After
    public void after() {
        if (this.componentUnderTest.isStarted()) {
            this.componentUnderTest.stop();
        }
    }

    protected final InputStream createPayloadInputStream(String payloadAsString) {
        return new ByteArrayInputStream(payloadAsString.getBytes());
    }

    protected ComponentConfiguration createComponentConfiguration(Logger logger) {
        return new ComponentConfiguration("componentUnderTest", logger);
    }

    protected abstract Component createComponentUnderTest(
            ComponentConfiguration componentConfiguration);

    protected abstract Message createMessageToProcessAsProvider(
            ServiceConfiguration testServiceConfiguration) throws Exception;

    @Test
    public void testIsStartedAfterConstruction() throws Exception {
        assertFalse(componentUnderTest.isStarted());
    }

    @Test
    public void testIsStartedAfterStart() throws Exception {
        componentUnderTest.start();
        assertTrue(componentUnderTest.isStarted());
    }

    @Test
    public void testIsStartedAfterStop() throws Exception {
        componentUnderTest.start();
        componentUnderTest.stop();
        assertFalse(componentUnderTest.isStarted());
    }

    @Test
    public void testInstallService() throws Exception {
        componentUnderTest.start();
        ServiceConfiguration serviceConfiguration = createTestProvideServiceConfiguration();
        assertFalse(componentUnderTest.isServiceInstalled(serviceConfiguration));
        componentUnderTest.installService(serviceConfiguration);
        assertTrue(componentUnderTest.isServiceInstalled(serviceConfiguration));
    }

    @Test
    public void testUnInstallService() throws Exception {
        componentUnderTest.start();
        ServiceConfiguration serviceConfiguration = createTestProvideServiceConfiguration();
        assertFalse(componentUnderTest.isServiceInstalled(serviceConfiguration));
        componentUnderTest.installService(serviceConfiguration);
        assertTrue(componentUnderTest.isServiceInstalled(serviceConfiguration));
        componentUnderTest.uninstallService(serviceConfiguration);
        assertFalse(componentUnderTest.isServiceInstalled(serviceConfiguration));
    }
    
    @Test
    public void testUnInstallAllServices() throws Exception {
        componentUnderTest.start();
        ServiceConfiguration serviceConfiguration = createTestProvideServiceConfiguration();
        assertFalse(componentUnderTest.isServiceInstalled(serviceConfiguration));
        componentUnderTest.installService(serviceConfiguration);
        assertTrue(componentUnderTest.isServiceInstalled(serviceConfiguration));
        componentUnderTest.uninstallAllServices();
        assertFalse(componentUnderTest.isServiceInstalled(serviceConfiguration));
    }
    
    @Test
    public void testNoMonitAfterStart() throws Exception {
        this.componentUnderTest.start();
        assertEquals(0, this.testHandler.getAllRecords(Level.MONIT).size());
    }

    protected abstract ServiceConfiguration createTestProvideServiceConfiguration()
            throws Exception;

    protected ServiceConfiguration createTestProvideServiceBaseConfiguration() throws Exception {
        return createTestServiceBaseConfiguration("testProvideService", ServiceType.PROVIDE);
    }

    protected ServiceConfiguration createTestServiceBaseConfiguration(final String configurationName,
            final ServiceType serviceType) throws Exception {
        ServiceConfiguration testInternalServiceConguration = new ServiceConfiguration(configurationName, new QName(
                "http://testProvideService", "testProvideInterfaceName"), new QName("http://testProvideService",
                "testServiceName"), "testEndpointName", serviceType);

        return testInternalServiceConguration;
    }

    @Test(timeout = 300000)
    public void testProcessMessageAsConsumerLogsNothingBecauseMonitDisabled() throws Exception {
        this.componentUnderTest.start();

        ServiceConfiguration providerServiceConfiguration = createTestProvideServiceConfiguration();
        this.componentUnderTest.installService(providerServiceConfiguration);

        ServiceConfiguration consumeServiceConfiguration = createTestConsumeServiceConfiguration();
        this.componentUnderTest.installService(consumeServiceConfiguration);

        this.logger.setLevel(Level.INFO);

        generateExternalEvent();

        this.componentUnderTest.pollRequestFromConsumer();

        List<LogRecord> records = this.testHandler.getAllRecords(Level.MONIT);
        assertEquals(0, records.size());
    }

    protected abstract ServiceConfiguration createTestConsumeServiceConfiguration()
            throws Exception;

    protected abstract Map<String, String> generateExternalEvent() throws Exception;

    protected void checkLoggedMonitRecordsOnBeginningOfProcessMessageAsConsumer(
            List<LogRecord> logRecords, ServiceConfiguration serviceConfiguration,
            Map<String, String> expectedSpecificParams) throws SocketException {

        assertEquals(1, logRecords.size());

        LogRecord record = logRecords.get(0);
        Object[] logParameters = record.getParameters();
        assertEquals(1, logParameters.length);

        assertTrue(logParameters[0] instanceof LogData);
        LogData logData = (LogData) logParameters[0];

        assertEquals(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN,
                logData.get(FlowLogData.FLOW_STEP_TRACE_CODE));

        assertFalse(StringHelper.isNullOrEmpty((String) logData
                .get(FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME)));
        assertFalse(StringHelper.isNullOrEmpty((String) logData
                .get(FlowLogData.FLOW_STEP_ID_PROPERTY_NAME)));

        assertEquals(serviceConfiguration.getInterfaceQName().toString(),
                logData.get(FlowLogData.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME));
        assertEquals(serviceConfiguration.getServiceQName().toString(),
                logData.get(FlowLogData.FLOW_STEP_SERVICE_NAME_PROPERTY_NAME));
        assertEquals(serviceConfiguration.getEndpointName(),
                logData.get(FlowLogData.FLOW_STEP_ENDPOINT_NAME_PROPERTY_NAME));

        for (String logDataKey : expectedSpecificParams.keySet()) {
            String expectedParamValue = expectedSpecificParams.get(logDataKey);
            assertEquals(expectedParamValue, logData.get(logDataKey));
        }
    }
}
